"""
eve images
"""

from allianceauth.eveonline.evelinks.eveimageserver import (
    character_portrait_url,
    type_render_url,
)
from allianceauth.eveonline.models import EveCharacter


def get_character_portrait_from_evecharacter(
    character: EveCharacter, size: int = 32, as_html: bool = False
) -> str:
    """
    get the character portrait from EveCharacter model
    :param size:
    :param character:
    :param portrait_size:
    :param as_html:
    :return:
    """
    portrait_url = character_portrait_url(
        character_id=character.character_id, size=size
    )

    return_value = portrait_url

    if as_html is True:
        return_value = (
            '<img class="aasrp-character-portrait img-rounded" '
            'src="{portrait_url}" alt="{character_name}">'.format(
                portrait_url=portrait_url, character_name=character.character_name
            )
        )

    return return_value


def get_type_render_url_from_type_id(
    evetype_id: int, size: int = 32, evetype_name: str = None, as_html: bool = False
) -> str:
    """
    get type render from evetype_id
    :param evetype_id:
    :param size:
    :param evetype_name:
    :param as_html:
    :return:
    """
    render_url = type_render_url(type_id=evetype_id, size=size)

    return_value = render_url

    if as_html is True:
        alt_tag = ""
        if evetype_name is not None:
            alt_tag = f' alt="{evetype_name}"'

        return_value = (
            '<img class="aasrp-evetype-icon img-rounded" '
            'src="{render_url}"{alt_tag}>'.format(
                render_url=render_url, alt_tag=alt_tag
            )
        )

    return return_value



"""
Update DB relations
"""

from django.core.management.base import BaseCommand

from eveuniverse.models import EveType

from aasrp.managers import AaSrpManager
from aasrp.models import AaSrpRequest


def get_input(text):
    """
    wrapped input to update DB relations
    """

    return input(text)


class Command(BaseCommand):
    help = "Update DB Relations"

    def _update_relations(self) -> None:
        """
        updating relations in database
        :return:
        """

        srp_requests = AaSrpRequest.objects.filter(ship=None)

        self.stdout.write(
            self.style.WARNING(
                "{count} SRP requests need to be updated".format(
                    count=srp_requests.count()
                )
            )
        )

        for srp_request in srp_requests:
            self.stdout.write(
                "Updating SRP request {request_code}".format(
                    request_code=srp_request.request_code
                )
            )

            try:
                srp_request__ship = EveType.objects.get(name=srp_request.ship_name)
            except EveType.DoesNotExist:
                srp_kill_link = AaSrpManager.get_kill_id(srp_request.killboard_link)

                (ship_type_id, ship_value, victim_id) = AaSrpManager.get_kill_data(
                    srp_kill_link
                )

                (
                    srp_request__ship,
                    created_from_esi,
                ) = EveType.objects.get_or_create_esi(id=ship_type_id)

            srp_request.ship = srp_request__ship
            srp_request.save()

    def handle(self, *args, **options):
        """
        ask before running ...
        :param args:
        :param options:
        """

        self.stdout.write(
            "This will update the relations between various tables in the database."
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting migration. Please stand by.")
            self._update_relations()
            self.stdout.write(self.style.SUCCESS("Update complete!"))
        else:
            self.stdout.write(self.style.WARNING("Aborted."))



"""
constants
"""

from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from aasrp import __version__

VERBOSE_NAME = "AA-SRP - A ship replacement module for Alliance Auth"
USERAGENT = "{verbose_name} v{version} {github_url}".format(
    verbose_name=slugify(VERBOSE_NAME, allow_unicode=True),
    version=__version__,
    github_url="https://github.com/ppfeufer/aa-srp",
)
EVE_CATEGORY_ID_SHIP = 6

SRP_REQUEST_NOTIFICATION_INQUIRY_NOTE = _(
    "If you have any questions regarding your SRP request, feel free to contact your "
    "request reviser.\nPlease make sure to always add the SRP-Code and the "
    "Request-Code with your inquiry."
)

ZKILLBOARD_BASE_URL = "https://zkillboard.com/"
ZKILLBOARD_API_URL = "https://zkillboard.com/api/"
ZKILLBOARD_BASE_URL_REGEX = r"^http[s]?://zkillboard\.com/"
ZKILLBOAR_KILLMAIL_URL_REGEX = r"^http[s]?://zkillboard\.com/kill/\d+\/"



import re
from urllib.parse import urljoin

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.urls import reverse


# old: get_absolute_url
def reverse_absolute(viewname: str, args: list = None) -> str:
    """returns absolute URL for given url"""
    return urljoin(site_absolute_url(), reverse(viewname, args=args))


# TODO: Only enable for alliance auth
# old: get_site_base_url
def site_absolute_url() -> str:
    """return absolute URL for this Alliance Auth site"""
    try:
        match = re.match(r"(.+)\/sso\/callback", settings.ESI_SSO_CALLBACK_URL)
        if match:
            return match.group(1)
    except AttributeError:
        pass

    return ""


def static_file_absolute_url(file_path: str) -> str:
    """returns absolute URL to a static file

    Args:
        file_path: relative path to a static file
    """
    return urljoin(site_absolute_url(), staticfiles_storage.url(file_path))
